/*main.cxx
  Yury P. Fedorchenko 2005-2009
*/
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#include <Fl/Fl.H>
#include <Fl/Fl_Widget.H>
#include <Fl/Fl_Double_Window.H>
#include <Fl/Fl_Menu_Bar.H>
#include <Fl/Fl_Box.H>
#include <Fl/Fl_Output.H>

#include <Fl/fl_ask.H>
#include <Fl/fl_draw.H>
#include <Fl/x.H>


#include <stdlib.h>
#include <stdio.h>
#ifndef _MSC_VER
#include <strings.h>
#endif
//
#include "gm.h"
#include "game.h"
#include "intro.h"
#include "network.h"

#if !defined(WIN32) && !defined(__APPLE__)
#include "X11/xpm.h"
#include "flmanowar.xpm"
#endif

#include "nls.h"
#include <iostream>
namespace {
struct locinit{
  locinit(){
#ifdef ENABLE_NLS
//  const char* btd=bindtextdomain(GETTEXT_PACKAGE,PACKAGE_LOCALE_DIR);
//  const char* tdc=bind_textdomain_codeset(GETTEXT_PACKAGE,"UTF-8");
  setlocale(LC_ALL,"");
  std::cerr<<MNW_LOC_DIR<<'\t'<<PACKAGE<<std::endl;
  const char* btd=bindtextdomain(PACKAGE,MNW_LOC_DIR);
  const char* tdc=bind_textdomain_codeset(PACKAGE,"UTF-8");
  textdomain(PACKAGE);
#endif
  }
}aaaa;
};

Fl_Double_Window *mainWnd;
Fl_Box *drW;

gm_field *first_field;
gm_field *second_field;

Fl_Output *status;

void new_cb(Fl_Widget * o, long l)
{
  first_field->initialize();
  second_field->initialize();
  first_field->redraw();
  second_field->redraw();
  first_field->set_active();
  second_field->game_netstate = gm_field::AI;
}

void about_cb(Fl_Widget * o, void *)
{
  Fl_Window *oo = about();
  oo->show();
}
void rules_cb(Fl_Widget * o, void *)
{
  rules();
}

void quit_cb(Fl_Widget * o, void *)
{
  if (fl_ask(_("Do you realy wanna quit?")) == 1)
    exit(0);
}

void level_cb(Fl_Widget * o, void *)
{
  const char *otv = fl_input(_("level from 0 to 3 (curent %d)"), "", second_field->level);
  if (otv) {
    int a = atoi(otv);
    second_field->level = a;
    first_field->level = a;
  }
}

Fl_Menu_ *menu_;
Fl_Menu_Bar *mainMenu;
Fl_Menu_Item menItm[] = {
  {_("&Game"), 0, 0, 0, FL_SUBMENU},
  {_("&New"), 0, (Fl_Callback *) new_cb, 0, FL_MENU_DIVIDER},
//  {"&Level",0,level_cb,0,FL_MENU_DIVIDER},
  {_("&Quit"), FL_ALT + 'x', quit_cb, 0},
  {0},
#ifdef HAVE_LIBGLOOX
  {_("Network"), 0, 0, 0, FL_SUBMENU},
  {_("Connect"), 0, cb_connect, 0, FL_MENU_DIVIDER},
  {_("ContactList"), 0, cb_network_window, 0, FL_MENU_DIVIDER},
  {_("Setup"), 0, cb_network_setup},
  {0},
#endif
  {"&?", 0, 0, 0, FL_SUBMENU},
  {_("Rules"), 0, rules_cb, 0},
  {_("&About"), 0, about_cb, 0},
  {0},
  {0}
};


char * docdir;


int main(int argc, char **argv)
{
#ifdef ENABLE_NLS
//  const char* btd=bindtextdomain(GETTEXT_PACKAGE,PACKAGE_LOCALE_DIR);
//  const char* tdc=bind_textdomain_codeset(GETTEXT_PACKAGE,"UTF-8");
  setlocale(LC_ALL,"");
  textdomain(PACKAGE);
#endif
  fl_font(0, 10);
  intro();
#ifndef WIN32  
  docdir=strdup(DOCDIR);
#else
  docdir=strdup("./doc");
#endif  
  mainWnd = new Fl_Double_Window(Fl::w() / 2 - 300, Fl::h() / 2 - 200, 600, 350, "flManowar");
  mainMenu = new Fl_Menu_Bar(0, 0, 600, 25);
  mainMenu->menu(menItm);
  //mainMenu->box(FL_FLAT_BOX);
  mainMenu->textsize(12);

  mainWnd->add(mainMenu);
  drW = new Fl_Box(2, 25, 596, 340);
  //drW->box(FL_PLASTIC_DOWN_BOX);
  mainWnd->add(drW);
  first_field = new gm_field(1, 10, 35, 280);
  second_field = new gm_field(2, 310, 35, 280);
  status = new Fl_Output(5, 325, 590, 20);
  //status->box(FL_PLASTIC_DOWN_BOX);

  mainWnd->add(first_field);
  mainWnd->add(second_field);

  mainWnd->callback(quit_cb);

#if defined( WIN32)
  mainWnd->icon(LoadIcon(fl_display, MAKEINTRESOURCE(IDICON)));
#elif !defined(__APPLE__)
  fl_open_display();            // needed if display has not been previously opened
  Pixmap p, mask;
  XpmCreatePixmapFromData(fl_display, DefaultRootWindow(fl_display), (char **) flmanowar_xpm, &p, &mask, NULL);
  mainWnd->icon((char *) p);
#endif
#ifdef HAVE_LIBGLOOX
  read_prefs();
#endif
  mainWnd->show(argc, argv);

  //first_field->set_active();
  Fl::lock();
  while (Fl::wait() > 0) {
    void *thmsg = Fl::thread_message();
    if (thmsg) {
      printf("th mess recieved!\n");
    }
  }
  return 0;
}
