/*network.cxx
  Yury P. Fedorchenko 2009
*/
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#include "network.h"

#ifdef HAVE_LIBGLOOX

#include "nls.h"

#include "gm.h"

#include "FL/Fl_Window.H"
#include "FL/Fl_Input.H"
#include "FL/Fl_Output.H"
#include "FL/Fl_Multiline_Output.H"
#include "FL/Fl_Text_Display.H"
#include "FL/Fl_Int_Input.H"
#include "FL/Fl_Choice.H"
#include "FL/Fl_Tabs.H"
#include "FL/Fl_Pixmap.H"
#include "FL/Fl_Select_Browser.H"
#include "FL/Fl_Check_Button.H"
#include "FL/Fl_Preferences.H"
#include "FL/fl_ask.H"
#include "FL/Fl.H"

#include <string>
#include <stdarg.h>

#include "flmanowar.xpm"

extern Fl_Output *status;

  /// for storing preferences
struct xmpp_prefs {
  std::string uname, domain, passwd;  //passwd don't saves in settings
  enum eproxy { NONE = 0, SOCKS5 = 1, HTTP = 2 } proxy;
  bool proxy_auth;
   std::string proxy_name, proxy_uname, proxy_passwd; //proxy passwd saves in settings
  int proxy_port;
  /// read prefs
  void read_settings() {
    Fl_Preferences xmpp(Fl_Preferences::USER, "fedorchenko.net", "flmanowar/xmpp");
    char path[1024];
     xmpp.getUserdataPath(path, 1023);
    char buf[128];
    int intb;
     xmpp.get("uname", buf, "", 128);
     uname = buf;
     xmpp.get("domain", buf, "", 128);
     domain = buf;
     xmpp.get("proxy", intb, NONE);
     proxy = (eproxy) (intb);
    if (proxy != NONE) {
      xmpp.get("proxy_name", buf, "", 128);
      proxy_name = buf;
      xmpp.get("proxy_port", proxy_port, 1080);
      xmpp.get("proxy_auth", intb, 0);
      proxy_auth = intb;
      if (proxy_auth) {
        xmpp.get("proxy_uname", buf, "", 128);
        proxy_uname = buf;
        xmpp.get("proxy_passwd", buf, "", 128);
        proxy_passwd = buf;
      }
    }
  }
  /// save prefs 
  void save_settings() {
    Fl_Preferences xmpp(Fl_Preferences::USER, "fedorchenko.net", "flmanowar/xmpp");
    xmpp.set("uname", uname.c_str());
    xmpp.set("domain", domain.c_str());
    xmpp.set("proxy", proxy);
    if (proxy != NONE) {
      xmpp.set("proxy_name", proxy_name.c_str());
      xmpp.set("proxy_port", proxy_port);
      xmpp.set("proxy_auth", proxy_auth);
      if (proxy_auth) {
        xmpp.set("proxy_uname", proxy_uname.c_str());
        xmpp.set("proxy_passwd", proxy_passwd.c_str());
      }
    }
  }
};

  //
namespace {
  struct xmpp_prefs mnw_xmpp, xmpp_tmp;
  Fl_Window *wnd_setup = (Fl_Window *) 0;
  Fl_Input *ipun = (Fl_Input *) 0;
  Fl_Input *ipas = (Fl_Input *) 0;
  Fl_Input *ipn = (Fl_Input *) 0;

  Fl_Check_Button *chb = (Fl_Check_Button *) 0;
  Fl_Int_Input *ipp = (Fl_Int_Input *) 0;
  //
  void ui_update() {
    if (xmpp_prefs::NONE == xmpp_tmp.proxy) {
      ipun->deactivate();
      ipas->deactivate();
      ipn->deactivate();
      ipp->deactivate();
      chb->deactivate();
    } else {
      ipn->activate();
      ipp->activate();
      chb->activate();
      if (xmpp_tmp.proxy_auth) {
        ipun->activate();
        ipas->activate();
      } else {
        ipun->deactivate();
        ipas->deactivate();
      }
    }
  }

  void cb_uname(Fl_Input * i, void *) {
    xmpp_tmp.uname = i->value();
  }
  void cb_domain(Fl_Input * i, void *) {
    xmpp_tmp.domain = i->value();
  }
  void cb_pname(Fl_Input * i, void *) {
    xmpp_tmp.proxy_name = i->value();
  }
  void cb_puname(Fl_Input * i, void *) {
    xmpp_tmp.proxy_uname = i->value();
  }
  void cb_ppasswd(Fl_Input * i, void *) {
    xmpp_tmp.proxy_passwd = i->value();
  }
  void cb_pport(Fl_Int_Input * i, void *) {
    xmpp_tmp.proxy_port = atoi(i->value());
  }

  void cb_proxy(Fl_Choice * c, void *) {
    xmpp_tmp.proxy = (xmpp_prefs::eproxy) c->value();
    ui_update();
  }
  void cb_pauth(Fl_Check_Button * b, void *) {
    xmpp_tmp.proxy_auth = b->value();
    ui_update();
  }
  void cb_ok(Fl_Widget * o, void *) {
    mnw_xmpp = xmpp_tmp;
    mnw_xmpp.save_settings();
    o->window()->hide();
  }
  void cb_close(Fl_Widget * o, void *) {
    o->window()->hide();
  }

};

void read_prefs()
{
  mnw_xmpp.read_settings();
}

void cb_network_setup(Fl_Widget *, void *)
{
  mnw_xmpp.read_settings();
  xmpp_tmp = mnw_xmpp;
  if (!wnd_setup) {
    wnd_setup = new Fl_Window(240, 280, _("Jabber Settings"));
    Fl_Input *iun = new Fl_Input(130, 5, 100, 25, _("User Name:"));
    iun->value(xmpp_tmp.uname.c_str());
    iun->callback((Fl_Callback *) cb_uname);
    Fl_Input *idm = new Fl_Input(130, 35, 100, 25, _("Domain:"));
    idm->value(xmpp_tmp.domain.c_str());
    idm->callback((Fl_Callback *) cb_domain);

    Fl_Choice *cp = new Fl_Choice(130, 65, 100, 25, _("Proxy type:"));
    cp->add("None|Socks5|Http");
    cp->callback((Fl_Callback *) cb_proxy);
    cp->value(xmpp_tmp.proxy);

    ipn = new Fl_Input(130, 95, 100, 25, _("Proxy name:"));
    ipn->value(xmpp_tmp.proxy_name.c_str());
    ipn->callback((Fl_Callback *) cb_pname);

    ipp = new Fl_Int_Input(130, 125, 100, 25, _("Proxy port:"));
    char str[64];
    sprintf(str, "%d", xmpp_tmp.proxy_port);
    ipp->value(str);
    ipp->callback((Fl_Callback *) cb_pport);

    chb = new Fl_Check_Button(10, 155, 180, 25, _("Proxy need auth"));
    chb->value(xmpp_tmp.proxy_auth);
    chb->callback((Fl_Callback *) cb_pauth);

    ipun = new Fl_Input(130, 185, 100, 25, _("Proxy user:"));
    ipun->value(xmpp_tmp.proxy_uname.c_str());
    ipun->callback((Fl_Callback *) cb_puname);
    ipas = new Fl_Input(130, 215, 100, 25, _("Proxy passwd:"));
    ipas->value(xmpp_tmp.proxy_passwd.c_str());
    ipas->callback((Fl_Callback *) cb_ppasswd);

    Fl_Button *ok_b = new Fl_Button(75, 250, 70, 25, _("Ok"));
    ok_b->callback(cb_ok);
    Fl_Button *close_b = new Fl_Button(150, 250, 70, 25, _("Cancel"));
    close_b->callback(cb_close);
    wnd_setup->end();
    ui_update();
    wnd_setup->set_modal();
  }
  wnd_setup->show();
}

#include "threads.h"


#include "gloox/client.h"
#include "gloox/messagesessionhandler.h"
#include "gloox/messageeventhandler.h"
#include "gloox/messageeventfilter.h"
#include "gloox/chatstatehandler.h"
#include "gloox/chatstatefilter.h"
#include "gloox/connectionlistener.h"
#include "gloox/disco.h"
#include "gloox/stanza.h"
#include "gloox/gloox.h"
#include "gloox/presencehandler.h"
#include "gloox/lastactivity.h"
#include "gloox/loghandler.h"
#include "gloox/logsink.h"
#include "gloox/connectiontcpclient.h"
#include "gloox/connectionsocks5proxy.h"
#include "gloox/connectionhttpproxy.h"
#include "gloox/messagehandler.h"
using namespace gloox;

#ifndef _WIN32
# include <unistd.h>
#endif

#include <stdio.h>
#include <string>

#if defined( WIN32 ) || defined( _WIN32 )
# include <windows.h>
#endif

class MnWrSession:public MessageSessionHandler, ConnectionListener, LogHandler, MessageEventHandler, MessageHandler, ChatStateHandler, PresenceHandler {
public:
  MnWrSession(): j(NULL),send_wnd(0) {
  } 
  virtual ~ MnWrSession() {
  }

  void start() {
    std::string connstr = mnw_xmpp.uname + "@" + mnw_xmpp.domain + "/flmanowar";
    JID jid(connstr);
    if (j)
      delete j;
    j = new Client(jid, mnw_xmpp.passwd);
    j->registerConnectionListener(this);
    j->registerMessageSessionHandler(this, 0);
    j->registerPresenceHandler(this);
    j->disco()->setIdentity("client", "flmanowar");
#ifndef _MSC_VER
    j->disco()->setVersion("messageTest", GLOOX_VERSION, "Linux");
    j->disco()->addFeature(XMLNS_CHAT_STATES);
#endif
    StringList ca;
    ca.push_back("/path/to/cacert.crt");
    j->setCACerts(ca);
    j->logInstance().registerLogHandler(LogLevelDebug, LogAreaAll, this);
    make_contact_wnd();
//
// this code connects to a jabber server through a SOCKS5 proxy
//
    if (xmpp_prefs::SOCKS5 == mnw_xmpp.proxy) {
      ConnectionSOCKS5Proxy *conn = new ConnectionSOCKS5Proxy(j, new ConnectionTCPClient(j->logInstance(),
                                                                                         mnw_xmpp.proxy_name, mnw_xmpp.proxy_port),
                                                              j->logInstance(), mnw_xmpp.domain);
      j->setConnectionImpl(conn);
    } else if (xmpp_prefs::HTTP == mnw_xmpp.proxy) {
      ConnectionTCPClient *conn0 = new ConnectionTCPClient(j->logInstance(), mnw_xmpp.proxy_name, mnw_xmpp.proxy_port);
/*      ConnectionSOCKS5Proxy* conn1 = new ConnectionSOCKS5Proxy( conn0, j->logInstance(), "old", 8080 );
        conn1->setProxyAuth( "socksuser", "sockspwd" );*/
      ConnectionHTTPProxy *conn = new ConnectionHTTPProxy(j, conn0, j->logInstance(), mnw_xmpp.domain);
      if (mnw_xmpp.proxy_auth) {
        conn->setProxyAuth(mnw_xmpp.proxy_uname, mnw_xmpp.proxy_passwd);
      }
      j->setConnectionImpl(conn);
    }
    //j->connect(true);
    if (j->connect(false)) {
      ConnectionError ce = ConnNoError;
      while (ce == ConnNoError) {
        ce = j->recv();
      }
      printf("ce: %d\n", ce);
    }
    fprintf(stderr, "flmanowar: start end\n");
  }

  virtual void onConnect() {
    printf("connected!!!\n");
  }

  virtual void onDisconnect(ConnectionError e) {
    fprintf(stderr, "flmanowar: disconnected: %d\n", e);
    if (e == ConnAuthenticationFailed)
      printf("auth failed. reason: %d\n", j->authError());
  }

  virtual bool onTLSConnect(const CertInfo & info) {
    printf("status: %d\nissuer: %s\npeer: %s\nprotocol: %s\nmac: %s\ncipher: %s\ncompression: %s\n"
           "from: %s\nto: %s\n", info.status, info.issuer.c_str(), info.server.c_str(), info.protocol.c_str(), info.mac.c_str(), info.cipher.c_str(), info.compression.c_str(), ctime((const time_t *) &info.date_from), ctime((const time_t *) &info.date_to));
    return true;
  }
  virtual void handlePresence(Stanza * stanza) {
    std::string fn;
    Presence p = stanza->presence();
    switch (p) {
    case (PresenceAvailable):
      fn = "@b@_";
      break;
    case (PresenceChat):
      fn = "@b@i";
      break;
    case (PresenceAway):
    case (PresenceXa):
      fn = "@i@c";
      break;
    case (PresenceDnd):
      fn = "@i@c";
      break;
      break;
    case (PresenceUnavailable):
      fn = "@r@-";
      break;
    default:
      fn = "@r@-";
    }
    fn += "@s@.";
    fn += stanza->from().bare();
    if (cnt_brz) {
      Fl::lock();
      int i;
      for (i = 1; i <= cnt_brz->size(); ++i) {
        const char *t = cnt_brz->text(i);
        printf("status:%s\t%s\n", stanza->from().bare().c_str(), &t[8]);
        if (strcmp(stanza->from().bare().c_str(), &t[8]) == 0) {
          cnt_brz->text(i, fn.c_str());
          break;
        }
      }
      //printf("i==%d",i,
      if (i > cnt_brz->size())
        cnt_brz->add(fn.c_str());
      Fl::unlock();
      Fl::awake();
    }
  }
  static void awake_cb(MessageSession * session) {
    if (fl_ask(_("Do you wanna play with %s?"), session->target().full().c_str())) {
      second_field->game_netstate = gm_field::ACCEPT;
      second_field->ses = session;
      srand(time(NULL));
      int gls = rand() % 256;
      char ms[256];
      sprintf(ms, "#MNW#accept#%d", gls);
      first_field->initialize();
      second_field->initialize();
      first_field->redraw();
      second_field->redraw();
      first_field->set_active();
      session->send(ms);
    } else
      session->send("#MNW#dontwant");
  }
  static void end_cb( /*const */ char *s) {
    fl_alert(s);
  }
  //
  virtual void handleMessage(Stanza * stanza, MessageSession * session) {
    printf("type: %d, subject: %s, message: %s, thread id: %s\n", stanza->subtype(), stanza->subject().c_str(), stanza->body().c_str(), stanza->thread().c_str());
    if (stanza->body().substr(0, 5) == "#MNW#") {
      MsgStatus("<%s>:%s", stanza->from().full().c_str(), stanza->body().c_str());
      std::string mnwmsg = stanza->body().substr(5);
      if (mnwmsg == "reject" || mnwmsg.substr(0, 5) == "error") {
        second_field->game_netstate = gm_field::NONE;
        MsgStatus("<%s>:%s(%s)", stanza->from().bare().c_str(), _("protocol error"),mnwmsg.c_str());
        return;
      }
      if (gm_field::NONE == second_field->game_netstate) {
        if (mnwmsg == "invite") {
          /*INVITE STATE */
          Fl::awake((void (*)(void *)) awake_cb, session);
        } else {
          session->send("#MNW#error_none");
        }
      } else if (gm_field::INVITE == second_field->game_netstate) {
        if (mnwmsg.substr(0, 6) == "accept") {
          second_field->game_netstate = gm_field::ACCEPT;
          first_field->set_active();
          srand(time(NULL));
          int gls = rand() % 256;
          if (atoi(mnwmsg.substr(6).c_str()) < gls) {
            session->send("#MNW#myhit");
            second_field->game_netstate = gm_field::SEND;
          } else {
            session->send("#MNW#yourhit");
            second_field->game_netstate = gm_field::WAIT;
          }
          /*ACCEPT STATE */
        } else {
          /*NONE STATE we dont send error in this point */
          second_field->game_netstate = gm_field::NONE;
          MsgStatus("<%s>:%s", stanza->from().full().c_str(), _("Don't want to play"));
        }
      } else if (gm_field::ACCEPT == second_field->game_netstate) {
        if (mnwmsg == "yourhit") {  //implement shoot order
          /* SEND STATE */
          second_field->game_netstate = gm_field::SEND;
        } else if (mnwmsg == "myhit") {
          /* WAIT STATE */
          second_field->game_netstate = gm_field::WAIT;
        } else {
          session->send("#MNW#error_accept");
        }
      } else if (gm_field::SEND == second_field->game_netstate) {
        if (mnwmsg == "hit") {
          /*continue shooting */
          //second_field->shoot[second_field->net_pnt.x][second_field->net_pnt.y]=1;
          second_field->ship[second_field->net_pnt.x][second_field->net_pnt.y] = 1;
          second_field->already_shoot = false;
          --second_field->ship_size;
          Fl::lock();
          second_field->redraw();
          Fl::unlock();
          Fl::awake();
          if (second_field->ship_size == 0) {
            second_field->game_netstate = gm_field::NONE;
            session->send("#MNW#iwin");
            Fl::awake((void (*)(void *)) end_cb, (void *) _("Winner!"));
          }
        } else if (mnwmsg == "miss") {
          /* WAIT STATE */
          second_field->already_shoot = false;
          second_field->game_netstate = gm_field::WAIT;
        } else {
          session->send("#MNW#error_send");
        }
      } else if (gm_field::WAIT == second_field->game_netstate) {
        //**"0:5"**//
        if (mnwmsg == "iwin") {
          second_field->game_netstate = gm_field::NONE;
          Fl::awake((void (*)(void *)) end_cb, (void *) _("Looser!") );
        } else if (mnwmsg.find_first_of(":") != mnwmsg.npos) {
          pnt x;
          x.x = atoi(mnwmsg.c_str());
          x.y = atoi(mnwmsg.substr(mnwmsg.find_first_of(":") + 1).c_str());
          /*find ship at first field and say what */
          first_field->shoot[x.x][x.y] = 1;
          if (first_field->ship[x.x][x.y] != 0) {
            --first_field->ship_size;
            session->send("#MNW#hit");
          } else {
            second_field->game_netstate = gm_field::SEND;
            session->send("#MNW#miss");
          }
          Fl::lock();
          first_field->redraw();
          Fl::unlock();
          Fl::awake();
        } else {
          session->send("#MNW#error_shoot");
        }
      } else {
        session->send("#MNW#reject");
      }
    } else {
      Fl::lock();
      for (int cur = 1; cur <= cnt_brz->size(); ++cur) {
        MSGWnd *mw = (MSGWnd *) cnt_brz->data(cur);
        if (mw && mw->ss == session) {
          mw->add_message("<%s>:%s", stanza->from().full().c_str(), stanza->body().c_str());
          break;
        }
      }
      Fl::unlock();
      Fl::awake();
    }
/*      std::string msg = "You said:\n> " + stanza->body() + "\nI like that statement.";
      std::string sub;
      if( !stanza->subject().empty() )
        sub = "Re: " +  stanza->subject();

      m_messageEventFilter->raiseMessageEvent( MessageEventDisplayed );
#if defined( WIN32 ) || defined( _WIN32 )
      Sleep( 1000 );
#else
      sleep( 1 );
#endif
      m_messageEventFilter->raiseMessageEvent( MessageEventComposing );
      m_chatStateFilter->setChatState( ChatStateComposing );
#if defined( WIN32 ) || defined( _WIN32 )
      Sleep( 2000 );
#else
      sleep( 2 );
#endif
      m_session->send( msg, sub );

      if( stanza->body() == "quit" )
        j->disconnect();*/
  }

  virtual void handleMessageEvent(const JID & from, MessageEventType event) {
    printf("received event: %d from: %s\n", event, from.full().c_str());
  }

  virtual void handleChatState(const JID & from, ChatStateType state) {
    printf("received state: %d from: %s\n", state, from.full().c_str());
  }

  virtual void handleMessageSession(MessageSession * session) {
    std::string incjid = session->target().bare();
    int cur = 1;
    for (; cur <= cnt_brz->size(); ++cur) {
      if (strcmp(incjid.c_str(), &cnt_brz->text(cur)[8]) == 0) {
        make_message_wnd(&cnt_brz->text(cur)[8], session, cur);
        break;
      }
    }
    if (cur > cnt_brz->size()) {
      std::string as = "@r@-@s@.";
      as += incjid;
      cnt_brz->add(as.c_str());
      make_message_wnd(&cnt_brz->text(cur)[8], session, cur);
    }
  }

  virtual void handleLog(LogLevel level, LogArea area, const std::string & message) {
    //printf("log: level: %d, area: %d, %s\n", level, area, message.c_str() );
  }

private:
  Client * j;
public:
  Client * client() {
    return j;
  }
  Fl_Window *contact_wnd;
  Fl_Browser *cnt_brz;
  Fl_Choice *ch_stat;
  Fl_Window *send_wnd;
  Fl_Tabs *send_tabs;
  /// window for supporting message session
  class MSGWnd:public Fl_Group {
    MessageEventFilter *mEF;
    ChatStateFilter *cSF;
  public:
    MessageSession * ss;
    Fl_Input *in;
    Fl_Text_Display *out;
    static void inp_cb(Fl_Input * i, MessageSession * s) {
      s->send(i->value());
      ((MSGWnd *) i->parent())->out->buffer()->append(i->window()->label());
      ((MSGWnd *) i->parent())->out->buffer()->append(":");
      ((MSGWnd *) i->parent())->out->buffer()->append(i->value());
      ((MSGWnd *) i->parent())->out->buffer()->append("\n");
      i->value("");
    } static void invite_cb(Fl_Widget * i, MessageSession * s) {
      second_field->type_ = 2;
      second_field->game_netstate = gm_field::INVITE;
      second_field->ses = s;
      //new_cb(i, 1L);
      first_field->initialize();
      second_field->initialize();
      Fl::lock();
      first_field->redraw();
      second_field->redraw();
      Fl::unlock();
      s->send("#MNW#invite");
    }
    MSGWnd(const char *jidstr, MessageSession * S):Fl_Group(4, 4, 302, 150, jidstr), ss(S) {
      out = new Fl_Text_Display(4, 4, 300, 120);
      out->textsize(10);
      out->buffer(new Fl_Text_Buffer());
      in = new Fl_Input(4, 125, 270, 25);
      in->textsize(10);
      in->when(FL_WHEN_ENTER_KEY);
      in->callback((Fl_Callback *) inp_cb, ss);
      Fl_Button *bt = new Fl_Button(276, 125, 25, 25);
      bt->image((new Fl_Pixmap(flmanowarbt_xpm)));
      bt->callback((Fl_Callback *) invite_cb, ss);
      resizable(out);
      end();
      ss->registerMessageHandler(MnWrSession::MNWsession);
      mEF = new MessageEventFilter(ss);
      mEF->registerMessageEventHandler(MnWrSession::MNWsession);
      cSF = new ChatStateFilter(ss);
      cSF->registerChatStateHandler(MnWrSession::MNWsession);
    }
    //void show();
    /// add message
    void add_message(const char *fmt, ...) {
      printf("add message\n");
      va_list ap;
      va_start(ap, fmt);
      char buffer[1024];
      vsprintf(buffer, fmt, ap);
      va_end(ap);
      Fl::lock();
      out->buffer()->append(buffer);
      out->buffer()->append("\n");
      parent()->redraw();
      Fl::unlock();
      Fl::awake((Fl_Awake_Handler) shwnd_cb, window());
      //printf("add stat:%s\n", buffer);
    }
  };
private:
  /// users list callback
  static void cb_brz(Fl_Select_Browser * b, void *o) {
    int cur = b->value();
    if (cur != 0) {
      MSGWnd *mw = (MSGWnd *) b->data(cur);
      if (!mw) {
        MessageSession *S = new MessageSession(MnWrSession::MNWsession->client(), JID(&b->text(cur)[8]));
        MnWrSession::MNWsession->make_message_wnd(&b->text(cur)[8], S, cur);
      } else {
        Fl::awake((Fl_Awake_Handler) shwnd_cb, mw->window());
      }
    }
  }
  /// write message to status bar
  int MsgStatus(const char *fmt, ...) {
    va_list ap;
    va_start(ap, fmt);
    char buffer[1024];
    vsprintf(buffer, fmt, ap);
    va_end(ap);
    Fl::lock();
    status->value(buffer);
    Fl::unlock();
    Fl::awake();
    printf("Msg stat:%s\n", buffer);
    return 1;
  }
  /// create contact window
  static void shwnd_cb(void *w) {
    Fl_Window *wnd = (Fl_Window *) w;
    wnd->show();
  }
  void make_contact_wnd() {
    Fl::lock();
    char str[1024];
    sprintf(str, "C:%s", j->jid().full().c_str());
    contact_wnd = new Fl_Window(150, 320, str);
    cnt_brz = new Fl_Select_Browser(2, 2, 146, 293);
    cnt_brz->callback((Fl_Callback *) cb_brz, this);
    ch_stat = new Fl_Choice(2,297,146,21);
    ch_stat->add(_("Online|Away|Not Aviable|Offline"));
    contact_wnd->end();
    contact_wnd->set_non_modal();
    Fl::unlock();
    Fl::awake((Fl_Awake_Handler) shwnd_cb, contact_wnd);
  }
  /// create new tab in message window with given Message Session
  void make_message_wnd(const char *label, MessageSession * S, int cur) {
    int W(308), H(150);
    if (!send_wnd) {
      send_wnd = new Fl_Window(310, 180, j->jid().bare().c_str());
      send_tabs = new Fl_Tabs(2, 2, 308, 176);
      send_wnd->resizable(send_wnd);
      send_wnd->set_non_modal();
      send_tabs->end();
    } else {
      W = send_tabs->child(0)->w();
      H = send_tabs->child(0)->h();
    }
    //char labeltmp[1024];
    //sprintf(labeltmp,"@.%s",label);
    Fl::lock();
    MSGWnd *mw = new MSGWnd(label, S);
    mw->size(W, H);
    send_tabs->add(mw);
    cnt_brz->data(cur, mw);
    Fl::unlock();
    Fl::awake((Fl_Awake_Handler) shwnd_cb, send_wnd);
  }
public:
  static MnWrSession *MNWsession;
};


MnWrSession *MnWrSession::MNWsession;

namespace {
  Fl_Thread xmpp_th;
  void *xmpp_th_f(void *m) {
    MnWrSession::MNWsession->start();
    fprintf(stderr, "end ups\n");
    delete MnWrSession::MNWsession;
    MnWrSession::MNWsession = NULL;
    return 0L;
  }
}

void cb_connect(Fl_Widget *, void *)
{
  if (!MnWrSession::MNWsession) {
    char buf[1024];
    sprintf(buf, _("Enter password for:%s@@%s"), mnw_xmpp.uname.c_str(), mnw_xmpp.domain.c_str());
    const char *pass = fl_password(buf);
    if (pass) {
      MnWrSession::MNWsession = new MnWrSession();
      mnw_xmpp.passwd = pass;
      fl_create_thread(xmpp_th, xmpp_th_f, MnWrSession::MNWsession);
    }
  }
}


void cb_network_window(Fl_Widget *, void *)
{
  if (MnWrSession::MNWsession) {
    MnWrSession::MNWsession->contact_wnd->show();
  }
}
#endif
